local navconst = require "uzelok.issues.is1.uznav.uznav_const"
local dispatcher = require "crit.dispatcher"
local Button = require "crit.button"

local h_touch = hash("touch")
local action = { x = 0.0, y = 0.0, pressed = false, released = false, }

local M = {}

-- functions to resolve function references for variable message content
-- there is forbidden to send functions over msg.post() and size < 2Kb
local function get_subpage_level(self)
  return {subpage_level = self.navbtn.subpage_level}
end

local function no_message(self)
  return {}
end

local message_functions = {
  ["no_message"] = no_message,
  ["get_subpage_level"] = get_subpage_level,
}


function M.btn_action_shake (node, done_cb)
  --  if true then return end
  local done
  local scale = gui.get_scale(node)
  local initial_scale = gui.get_scale(node)

  local cb = function(self, node)
    gui.set_scale(node, initial_scale)
    if done then
      done(self, node)
    end
  end
  gui.cancel_animation(node, "scale.x")
  gui.cancel_animation(node, "scale.y")
  ---gui.set_scale(node, initial_scale)
  gui.set_scale(node, scale * 1.2)
  gui.animate(node, "scale.x", scale.x, gui.EASING_OUTELASTIC, 0.8)

  done = done_cb
  gui.animate(node, "scale.y", scale.y, gui.EASING_OUTELASTIC, 0.8, 0.05, cb)
end

local function on_msg_set_button_state(self, message_id, message)
  if message_id == navconst.MSG_SET_BUTTON_STATE and message.button == self.navbtn.props.name then

    if message.to_state == navconst.STATE_DISABLED and self.navbtn.state == navconst.STATE_ENABLED then
      print(self.navbtn.props.name, " goes to disabled")
      self.navbtn.state = navconst.STATE_DISABLED
      pprint("self.navbtn.btn_bkg", self.navbtn.btn_bkg)
      self.navbtn.btn_bkg.set_enabled( false)
      self.navbtn.btn_glyph.set_enabled( false)

    elseif message.to_state == navconst.STATE_ENABLED and self.navbtn.state == navconst.STATE_DISABLED then
      print(self.navbtn.props.name, " goes to enabled")
      self.navbtn.state = navconst.STATE_ENABLED
      self.navbtn.btn_bkg.set_enabled( true)
      self.navbtn.btn_glyph.set_enabled( true)

    end
  end
end

local function on_msg_set_subpage_level(self, message_id, message)
  if message_id == navconst.MSG_SET_SUBPAGE_LEVEL
  and message.button == self.navbtn.props.name
  and message.subpage_level then
    print(self.navbtn.props.name, " new subpage_level = "..message.subpage_level )
    self.navbtn.subpage_level = message.subpage_level
  end
end

local function on_msg_press_button(self, message_id, message)
  if message_id == navconst.MSG_PRESS_BUTTON
  and message.button == self.navbtn.props.name
  and self.navbtn.state == navconst.STATE_ENABLED then
    print("message press_button for button ".. message.button )

    local node_pos = gui.get_position(gui.get_node(self.navbtn.props.name))
    action.x = node_pos.x action.y = node_pos.y

    -- press, need for release
    action.pressed = true action.released = false
    self.navbtn.btn_bkg.on_input(h_touch, action)
    self.navbtn.btn_glyph.on_input(h_touch, action)

    -- release, here action-function will be called
    action.pressed = false action.released = true
    self.navbtn.btn_bkg.on_input(h_touch, action)
    self.navbtn.btn_glyph.on_input(h_touch, action)

    -- disable hover state
    action.x = 0 action.y = 0
    action.pressed = false action.released = false
    self.navbtn.btn_bkg.on_input(nil, action)
    self.navbtn.btn_glyph.on_input(nil, action)

  end
end

local function on_msg_set_button_design(self, message_id, message)

  if message_id == navconst.MSG_SET_BUTTON_DESIGN and message.button == self.btn.btn_name then
    print("message set_button_design for button ".. message.button )

    local design_no = message.design_no or self.navbtn.props.init_design

    local btn_bkg_txtrs = self.navbtn.props.textures.bkg[design_no]
    if btn_bkg_txtrs then
      self.navbtn.btn_bkg.on_state_change = Button.flipbook({
        [Button.STATE_HOVER] = hash(btn_bkg_txtrs.hover),
        [Button.STATE_DEFAULT] = hash(btn_bkg_txtrs.default),
        [Button.STATE_PRESSED] = hash(btn_bkg_txtrs.pressed),
        [Button.STATE_DISABLED] = hash(btn_bkg_txtrs.disabled),
      })
      self.navbtn.btn_bkg.refresh_state()
    end

    local btn_glyph_txtrs = self.navbtn.props.textures.glyph[design_no]
    if btn_glyph_txtrs then
      self.navbtn.btn_glyph.on_state_change = Button.flipbook({
        [Button.STATE_HOVER] = hash(btn_glyph_txtrs.hover),
        [Button.STATE_DEFAULT] = hash(btn_glyph_txtrs.default),
        [Button.STATE_PRESSED] = hash(btn_glyph_txtrs.pressed),
        [Button.STATE_DISABLED] = hash(btn_glyph_txtrs.disabled),
      })
      self.navbtn.btn_glyph.refresh_state()
    end
  end
end

local function on_msg_init_button(self, message_id, message)
  if message_id == navconst.MSG_INIT_BUTTON and message.button == self.btn.btn_name then

    self.navbtn = {}
    assert(message.btn_props, "Button properties is not found in the message ".. message_id)

    self.navbtn.props = message.btn_props
    self.navbtn.state = self.navbtn.props.init_state
    self.navbtn.subpage_level = 0

    -- pprint("self.navbtn.props", self.navbtn.props)
    -- print("hoover", self.navbtn.props.textures.hoover)
    local btn_bkg_txtrs = self.navbtn.props.textures.bkg[self.navbtn.props.init_design or 1]
    gui.set_texture(self.btn.n_bkg, self.navbtn.props.atlas)
    gui.play_flipbook(self.btn.n_bkg, btn_bkg_txtrs.default)

    self.navbtn.btn_bkg = Button.new(
      self.btn.n_bkg, {
        starts_enabled = self.navbtn.state == navconst.STATE_ENABLED,
        on_state_change = Button.flipbook({
          [Button.STATE_HOVER] = hash(btn_bkg_txtrs.hover),
          [Button.STATE_DEFAULT] = hash(btn_bkg_txtrs.default),
          [Button.STATE_PRESSED] = hash(btn_bkg_txtrs.pressed),
          [Button.STATE_DISABLED] = hash(btn_bkg_txtrs.disabled),
        }),
        action = function ()
          print(self.navbtn.props.name .." bkg pressed");
          print("Texture", gui.get_texture(self.btn.n_bkg) )
          M.btn_action_shake(self.btn.n_btn)
        end,
    })

    gui.set_texture(self.btn.n_glyph, self.navbtn.props.atlas)
    local btn_glyph_txtrs = self.navbtn.props.textures.glyph[self.navbtn.props.init_design or 1]
    self.navbtn.btn_glyph = Button.new(
      self.btn.n_glyph, {
        starts_enabled = self.navbtn.state == navconst.STATE_ENABLED,
        on_state_change = Button.flipbook({
          [Button.STATE_HOVER] = hash(btn_glyph_txtrs.hover),
          [Button.STATE_DEFAULT] = hash(btn_glyph_txtrs.default),
          [Button.STATE_PRESSED] = hash(btn_glyph_txtrs.pressed),
          [Button.STATE_DISABLED] = hash(btn_glyph_txtrs.disabled),
        }),
        action = function ()
          print(self.navbtn.props.name .." glyph pressed");
          dispatcher.dispatch(
            self.navbtn.props.message_to_send.message_id,
            message_functions[self.navbtn.props.message_to_send.message](self)
          )
        end,
      }
    )

    if self.sub_id then dispatcher.unsubscribe(self.sub_id) end
    self.sub_id = dispatcher.subscribe(self.navbtn.props.messages_to_subscribe)
  end
end

function M.on_message(self, message_id, message)

  if message_id == navconst.MSG_PRESS_BUTTON then
    on_msg_press_button(self, message_id, message)

  elseif message_id == navconst.MSG_SET_SUBPAGE_LEVEL then
    on_msg_set_subpage_level(self, message_id, message)

  elseif message_id == navconst.MSG_SET_BUTTON_STATE then
    on_msg_set_button_state(self, message_id, message)

  elseif message_id == navconst.MSG_SET_BUTTON_DESIGN then
    on_msg_set_button_design(self, message_id, message)

  elseif message_id == navconst.MSG_INIT_BUTTON then
    on_msg_init_button(self, message_id, message)

  else
    print("uznav_btn.on_message: unhandled message ", message_id)
  end

end

return M
